@@ -100,7 +100,9 @@ class LatexUnicodeTable:
         """
         # TODO complete this list
         # register special symbols
-        self.register(u' ', b'\\ ', encode=False)  # encoding left to lexer
+        self.register(u'\n\n', b' \\par', encode=False)
+        self.register(u'\n\n', b'\\par', encode=False)
+        self.register(u' ', b'\\ ', encode=False)
         self.register(u'\N{EN DASH}', b'--')
         self.register(u'\N{EN DASH}', b'\\textendash')
         self.register(u'\N{EM DASH}', b'---')
@@ -237,8 +239,8 @@ class LatexUnicodeTable:
         self.register(u'\N{LATIN SMALL LETTER A WITH MACRON}', b'\\=a')
         self.register(u'\N{LATIN CAPITAL LETTER A WITH BREVE}', b'\\u A')
         self.register(u'\N{LATIN SMALL LETTER A WITH BREVE}', b'\\u a')
-        self.register(u'\N{LATIN CAPITAL LETTER A WITH OGONEK}', b'\\c A')
-        self.register(u'\N{LATIN SMALL LETTER A WITH OGONEK}', b'\\c a')
+        self.register(u'\N{LATIN CAPITAL LETTER A WITH OGONEK}', b'\\k A')
+        self.register(u'\N{LATIN SMALL LETTER A WITH OGONEK}', b'\\k a')
         self.register(u'\N{LATIN CAPITAL LETTER C WITH ACUTE}', b"\\'C")
         self.register(u'\N{LATIN SMALL LETTER C WITH ACUTE}', b"\\'c")
         self.register(u'\N{LATIN CAPITAL LETTER C WITH CIRCUMFLEX}', b'\\^C')
@@ -255,8 +257,8 @@ class LatexUnicodeTable:
         self.register(u'\N{LATIN SMALL LETTER E WITH BREVE}', b'\\u e')
         self.register(u'\N{LATIN CAPITAL LETTER E WITH DOT ABOVE}', b'\\.E')
         self.register(u'\N{LATIN SMALL LETTER E WITH DOT ABOVE}', b'\\.e')
-        self.register(u'\N{LATIN CAPITAL LETTER E WITH OGONEK}', b'\\c E')
-        self.register(u'\N{LATIN SMALL LETTER E WITH OGONEK}', b'\\c e')
+        self.register(u'\N{LATIN CAPITAL LETTER E WITH OGONEK}', b'\\k E')
+        self.register(u'\N{LATIN SMALL LETTER E WITH OGONEK}', b'\\k e')
         self.register(u'\N{LATIN CAPITAL LETTER E WITH CARON}', b'\\v E')
         self.register(u'\N{LATIN SMALL LETTER E WITH CARON}', b'\\v e')
         self.register(u'\N{LATIN CAPITAL LETTER G WITH CIRCUMFLEX}', b'\\^G')
@@ -278,8 +280,8 @@ class LatexUnicodeTable:
         self.register(u'\N{LATIN CAPITAL LETTER I WITH BREVE}', b'\\u I')
         self.register(u'\N{LATIN SMALL LETTER I WITH BREVE}', b'\\u\\i')
         self.register(u'\N{LATIN SMALL LETTER I WITH BREVE}', b'\\u i')
-        self.register(u'\N{LATIN CAPITAL LETTER I WITH OGONEK}', b'\\c I')
-        self.register(u'\N{LATIN SMALL LETTER I WITH OGONEK}', b'\\c i')
+        self.register(u'\N{LATIN CAPITAL LETTER I WITH OGONEK}', b'\\k I')
+        self.register(u'\N{LATIN SMALL LETTER I WITH OGONEK}', b'\\k i')
         self.register(u'\N{LATIN CAPITAL LETTER I WITH DOT ABOVE}', b'\\.I')
         self.register(u'\N{LATIN SMALL LETTER DOTLESS I}', b'\\i')
         self.register(u'\N{LATIN CAPITAL LIGATURE IJ}', b'IJ', decode=False)
@@ -343,8 +345,8 @@ class LatexUnicodeTable:
             u'\N{LATIN CAPITAL LETTER U WITH DOUBLE ACUTE}',
             b'\\H U')
         self.register(u'\N{LATIN SMALL LETTER U WITH DOUBLE ACUTE}', b'\\H u')
-        self.register(u'\N{LATIN CAPITAL LETTER U WITH OGONEK}', b'\\c U')
-        self.register(u'\N{LATIN SMALL LETTER U WITH OGONEK}', b'\\c u')
+        self.register(u'\N{LATIN CAPITAL LETTER U WITH OGONEK}', b'\\k U')
+        self.register(u'\N{LATIN SMALL LETTER U WITH OGONEK}', b'\\k u')
         self.register(u'\N{LATIN CAPITAL LETTER W WITH CIRCUMFLEX}', b'\\^W')
         self.register(u'\N{LATIN SMALL LETTER W WITH CIRCUMFLEX}', b'\\^w')
         self.register(u'\N{LATIN CAPITAL LETTER Y WITH CIRCUMFLEX}', b'\\^Y')
@@ -385,8 +387,8 @@ class LatexUnicodeTable:
         self.register(u'\N{LATIN SMALL LETTER G WITH CARON}', b'\\v g')
         self.register(u'\N{LATIN CAPITAL LETTER K WITH CARON}', b'\\v K')
         self.register(u'\N{LATIN SMALL LETTER K WITH CARON}', b'\\v k')
-        self.register(u'\N{LATIN CAPITAL LETTER O WITH OGONEK}', b'\\c O')
-        self.register(u'\N{LATIN SMALL LETTER O WITH OGONEK}', b'\\c o')
+        self.register(u'\N{LATIN CAPITAL LETTER O WITH OGONEK}', b'\\k O')
+        self.register(u'\N{LATIN SMALL LETTER O WITH OGONEK}', b'\\k o')
         self.register(u'\N{LATIN SMALL LETTER J WITH CARON}', b'\\v\\j')
         self.register(u'\N{LATIN CAPITAL LETTER DZ}', b'DZ', decode=False)
         self.register(
@@ -433,7 +435,7 @@ class LatexUnicodeTable:
         self.register(u'\N{BREVE}', b'\\u{}')
         self.register(u'\N{DOT ABOVE}', b'\\.{}')
         self.register(u'\N{RING ABOVE}', b'\\r{}')
-        self.register(u'\N{OGONEK}', b'\\c{}')
+        self.register(u'\N{OGONEK}', b'\\k{}')
         self.register(u'\N{SMALL TILDE}', b'\\~{}')
         self.register(u'\N{DOUBLE ACUTE ACCENT}', b'\\H{}')
         self.register(u'\N{LATIN SMALL LIGATURE FI}', b'fi', decode=False)
@@ -543,6 +545,10 @@ class LatexUnicodeTable:
             u'\N{TRADE MARK SIGN}',
             b'\\texttrademark',
             package='textcomp')
+        # \=O and \=o will be translated into Ō and ō before we can
+        # match the full latex string... so decoding disabled for now
+        self.register(u'Ǭ', br'\textogonekcentered{\=O}', decode=False)
+        self.register(u'ǭ', br'\textogonekcentered{\=o}', decode=False)
 
     def register(self, unicode_text, latex_text, mode='text', package=None,
                  decode=True, encode=True):
@@ -558,7 +564,6 @@ class LatexUnicodeTable:
         :param bool encode: Whether this translation applies to encoding
             (default: ``True``).
         """
-        assert len(unicode_text) == 1
         if package is not None:
             # TODO implement packages
             pass
@@ -569,6 +574,8 @@ class LatexUnicodeTable:
             # XXX for the time being, we do not perform in-math substitutions
             return
         # tokenize, and register unicode translation
+        self.lexer.reset()
+        self.lexer.state = 'M'
         tokens = tuple(self.lexer.get_tokens(latex_text, final=True))
         if decode:
             if tokens not in self.unicode_map:
@@ -589,6 +596,7 @@ class LatexUnicodeTable:
                     self.max_length = max(self.max_length, len(alt_tokens))
                     self.unicode_map[alt_tokens] = u"{" + unicode_text + u"}"
         if encode and unicode_text not in self.latex_map:
+            assert len(unicode_text) == 1
             self.latex_map[unicode_text] = (latex_text, tokens)
 
 _LATEX_UNICODE_TABLE = LatexUnicodeTable(lexer.LatexIncrementalDecoder())
@@ -718,7 +726,7 @@ class LatexIncrementalDecoder(lexer.LatexIncrementalDecoder):
             # note: match is only possible at the *end* of the buffer
             # because all other positions have already been checked in
             # earlier iterations
-            for i in range(1, len(self.token_buffer) + 1):
+            for i in range(len(self.token_buffer), 0, -1):
                 last_tokens = tuple(self.token_buffer[-i:])  # last i tokens
                 try:
                     unicode_text = self.table.unicode_map[last_tokens]
