@@ -158,6 +158,27 @@ class TestDecoder(TestCase):
     def test_dz(self):
         self.decode(u"DZ", br'DZ')
 
+    def test_newline(self):
+        self.decode(u"hello world", b"hello\nworld")
+
+    def test_par1(self):
+        self.decode(u"hello\n\nworld", b"hello\n\nworld")
+
+    def test_par2(self):
+        self.decode(u"hello\n\nworld", b"hello\\par world")
+
+    def test_par3(self):
+        self.decode(u"hello\n\nworld", b"hello \\par world")
+
+    def test_ogonek1(self):
+        self.decode(u"ĄąĘęĮįǪǫŲų",
+                    br'\k A\k a\k E\k e\k I\k i\k O\k o\k U\k u')
+
+    def test_ogonek2(self):
+        # note: should decode into u"Ǭǭ" but can't support this yet...
+        self.decode(u"\\textogonekcentered {Ō}\\textogonekcentered {ō}",
+                    br'\textogonekcentered{\=O}\textogonekcentered{\=o}')
+
 
 class TestStreamDecoder(TestDecoder):
 
@@ -298,6 +319,23 @@ class TestEncoder(TestCase):
     def test_dz2(self):
         self.encode(u"Ǳ", br'DZ')
 
+    def test_newline(self):
+        self.encode(u"hello\nworld", b"hello\nworld")
+
+    def test_par1(self):
+        self.encode(u"hello\n\nworld", b"hello\n\nworld")
+
+    def test_par2(self):
+        self.encode(u"hello\\par world", b"hello\\par world")
+
+    def test_ogonek1(self):
+        self.encode(u"ĄąĘęĮįǪǫŲų",
+                    br'\k A\k a\k E\k e\k I\k i\k O\k o\k U\k u')
+
+    def test_ogonek2(self):
+        self.encode(u"Ǭǭ",
+                    br'\textogonekcentered{\=O}\textogonekcentered{\=o}')
+
 
 class TestStreamEncoder(TestEncoder):
 
